# -*- coding: utf-8 -*-
"""
This module contains Service implementation of RateYourMusic.
http://rateyourmusic.com
"""


from . import SchemaOrgService


##############################################################################


class RateYourMusic(SchemaOrgService):
    """
    Implementation of Service which is intended to parse RateYourMusic.
    Yes, because I can.
    """
    pass
