# -*- coding: utf-8 -*-
"""
This module contains Service implementation of Discogs.
http://discogs.com
"""


from . import SchemaOrgService


##############################################################################


class Discogs(SchemaOrgService):
    """
    Implementation of Service which is intended to parse Discogs.
    """
    pass
