# -*- coding: utf-8 -*-
"""
This module contains Service implementation of AllMusic Guide service.
http://allmusic.com
"""


from . import SchemaOrgService


##############################################################################


class AllMusic(SchemaOrgService):
    """
    Implementation of Service which is intended to parse AMG.
    """
    pass
