# -*- coding: utf-8 -*-
"""
This module contains capitalization definitions for RYM in Python 3.
"""


###############################################################################


def capitalize(text):
    """
    Python 3 primitive stub version of text capitalization. Actually it does
    nothing serious since NLTK is not ported to Python 3 yet.
    """
    return text