# -*- coding: utf-8 -*-
"""
This module contains Service implementation of Jamendo.
http://jamendo.com
"""


from . import SchemaOrgService


##############################################################################


class Jamendo(SchemaOrgService):
    """
    Implementation of Service which is intended to parse Jamendo.
    """
    pass
